"""Linear algebra structures (currently only vectors) in Euclidean space"""


class NonPositiveValueError(ValueError):
    """ValueError where the value is non-positive"""

    def __init__(self, val):
        self.val = val
        super().__init__(f"The value is {val}")


class Vector:
    """2d vector in Euclidean space"""

    # Class has a separate namespace where we can store variables common for all objects
    dimension = 2

    def __init__(self, x, y):
        self.x = x
        self.y = y

    # Property decorator makes methods without parameters look like object variables from the outside
    @property
    def euclidean_norm(self):
        """Return the euclidean distance from origin which is equal to euclidean norm"""
        return (self.x**2 + self.y**2)**0.5

    # Static method decorator marks a method that is associated with a class rather than an object
    @staticmethod
    def zero_vector():
        """Return an instance of Vector with zero length"""
        return Vector(0, 0)

    def __eq__(self, other):
        return self.x == other.x and self.y == other.y

    def __add__(self, other):
        return Vector(self.x + other.x, self.y + other.y)

    def __mul__(self, other):
        return Vector(self.x * other, self.y * other)

    def __repr__(self):
        return f"Vector({self.x}, {self.y})"


class YVector(Vector):
    """2d vector in Euclidean space where x=0"""

    def __init__(self, y):
        super().__init__(0, y)


class PositiveYVector(YVector):
    """2d vector in Euclidean space where x=0 and y>0"""

    def __init__(self, y):
        if y <= 0:
            raise NonPositiveValueError(y)
        super().__init__(y)


if __name__ == '__main__':
    try:
        PositiveYVector(-1)
    except NonPositiveValueError:
        print("Our custom exception raised")
